/* ***** BEGIN LICENSE BLOCK *****
 * Licensed under Version: MPL 1.1/GPL 2.0/LGPL 2.1
 * Full Terms at http://mozile.mozdev.org/license.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Playsophy code.
 *
 * The Initial Developer of the Original Code is Playsophy
 * Portions created by the Initial Developer are Copyright (C) 2002-2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *		James A. Overton <james@overton.ca>
 *
 * ***** END LICENSE BLOCK ***** */

function specialInit(event)
{
	var specialObj;

	mozileSetVersionById('version');
	if ( "arguments" in window && window.arguments.length >= 1 ) {
		specialObj = window.arguments[0];
		specialEntry( specialObj.entry );
		specialPopulate( specialObj, specialObj.selectedIndex );
		specialMenuCreate( specialObj, specialObj.selectedIndex );
		specialResize();
	}else{
		// must have input data!
	}
}

function specialEntry( entry ) {
	var readonly = false;
	if ( entry == "none" ) {
		readonly = true;
	}
	document.getElementById("charCodeBox").readonly = readonly;
}

function specialMenuCreate( specialObj, n ) {
	var menu = document.getElementById("specialMenu");
	menu.removeAllItems();
	for ( var i=0; i<specialObj.specials.length; i++ ) {
		menu.appendItem(specialObj.specials[i].name);
	}
	menu.selectedIndex = n;
}

function specialSelect( menulist ) {
	specialPopulate( window.arguments[0], menulist.selectedIndex );
	specialResize();
	return true;
}


// specialObj : contains array of special char objects
// n : index of specials to use.
//
function specialPopulate( specialObj, n ) {
	var i, blocks, columns;

	// Check in range.
	if ( n >= specialObj.specials.length )
		n = 0;

	// Store selected index.
	specialObj.selectedIndex = n;	

	// Set up the maximum number of columns.
	columns = specialObj.specials[n].columns;
	specialInsertColumns( columns );

	blocks = specialObj.specials[n].specialblocks;
	for( i=0; i < blocks.length; i++ ) {
		if ( "characters" in blocks[i] ) {
			specialInsertChars( blocks[i].characters, columns );
		} else if ( "range" in blocks[i] ) {
			specialInsertRange( blocks[i].range );
		}			
	}
}

function specialResize() {
	// This bizarre hack is required to remove any unwanted scrollbars.
	// I would have thought setting "overflow: auto;" would remove scrollbars if they are not needed,
	// but this doesn't happen.
	//
	try {
		var characterGrid = document.getElementById("characterGrid");
		characterGrid.style.overflow = "hidden";
		self.sizeToContent();
		characterGrid.style.overflow = "auto";
		self.sizeToContent();
	} catch (e) {
	}
}


// Insert the required columns for the grid.
//
function specialInsertColumns( numcols ) {
	var grid, colsList, rowsList, columns, column, i, n;
	
	try {
		grid = document.getElementById("characterGrid");
		//Remove any existing <columns> and children.
		colsList = grid.getElementsByTagName("columns");
		for ( i=colsList.length; i>0; i-- ) {
			grid.removeChild(colsList[i-1]);
		}
		//Remove any existing <rows> and children.
		rowsList = grid.getElementsByTagName("rows");
		for ( i=rowsList.length; i>0; i-- ) {
			grid.removeChild(rowsList[i-1]);
		}
		//Create new columns and rows elements.
		columns = grid.appendChild( document.createElement("columns") );
		grid.appendChild( document.createElement("rows") );
		
		//Add the column elements.
		column = document.createElement("column");
		for (n=0; n<numcols; ++n) {
			columns.appendChild( column );
		}
	} catch (e) {
		var f = ["special.js","specialInsertColumns"];
		opener.mozileDebug(f,1,"Exception: "+e);
	}
}

// Insert characters from a character array.
// The characters must be numbers.
//
function specialInsertChars( charlist, wrap ) {
	var grid, rows, row, i, c, charInt; 

	try {
		grid = document.getElementById("characterGrid");
		rows = grid.getElementsByTagName("rows")[0];
		i=0;
		while( i < charlist.length ) {
			row = document.createElement("row");
			for( c=0; c<wrap; c++ ) {
				switch( charlist[i] ) {
					case "S" :
						specialInsertSpacer( row );
						break;
					default :
						charInt = parseInt( charlist[i] );
						if ( !isNaN(charInt) )
							specialInsertLabel(row, charInt);
				}
				++i;
				if ( i >= charlist.length )
					break;
			}
			rows.appendChild(row);
		}
	} catch (e) {
		var f = ["special.js","specialInsertChars"];
		opener.mozileDebug(f,1,"Exception: "+e);
	}
}

// Insert the characters from a single range definition.
//
function specialInsertRange( rangeObj ) {
	var grid, rows, row, i, c, start, end, wrap; 

	try {
		start = rangeObj.start;
		end = rangeObj.end;
		wrap = rangeObj.wrap;
		grid = document.getElementById("characterGrid");
		rows = grid.getElementsByTagName("rows")[0];
		i=start;
		while( i <= end ) {
			row = document.createElement("row");
			for( c=0; c<wrap; c++ ) {
				specialInsertLabel(row, i);
				if ( ++i > end )
					break;
			}
			rows.appendChild(row);
		}
	} catch (e) {
		var f = ["special.js","specialInsertRange"];
		opener.mozileDebug(f,1,"Exception: "+e);
	}
}

// Insert a single character.
// The character is appended to row as a label element.
//
function specialInsertLabel( row, charcode ) {
	try {
		var label = document.createElement("label");
		label.setAttribute('class','character');
		label.setAttribute('value',String.fromCharCode(charcode));
		label.setAttribute('code',charcode);
		label.setAttribute('onclick','characterSelected(this)');
		label.setAttribute('ondblclick','characterSelectedNow(this)');
		row.appendChild(label);
	} catch (e) {
		var f = ["special.js","specialInsertLabel"];
		opener.mozileDebug(f,1,"Exception: "+e);
	}
}

function specialInsertSpacer( row ) {
	try {
		row.appendChild(document.createElement("spacer"));
	} catch (e) {
		var f = ["special.js","specialInsertSpacer"];
		opener.mozileDebug(f,1,"Exception: "+e);
	}
}

// Click character.
//
function characterSelected(node) {
	var code = node.getAttribute('code');
	codeSelected(code);
}

// Double click character.
//
function characterSelectedNow(node) {
	var code = node.getAttribute('code');
	codeSelected(code);
	insertEntity();
}
	
// Need to check code is in a valid range???
//
function codeSelected(code) {
	var codeNum = parseInt(code);
	if ( !isNaN(codeNum) ) {
		var charCodeBox = document.getElementById("charCodeBox");
		charCodeBox.value = codeNum;
		var charCodeDemo = document.getElementById("charCodeDemo");
		charCodeDemo.value = String.fromCharCode(codeNum);
	}
}

function insertEntity() {
	var charCodeBox = document.getElementById("charCodeBox");
	var code = parseInt( charCodeBox.value );
	if ( !isNaN(code) ) {
		opener.mozileInsertEntity(code);
	}
}

